require "zip"
require "json"
require "kartograph"
require "resource_kit"
require "qrapi/version"
require "active_model"
require "active_support/all"

module QRAPI
  autoload :Client,                             "qrapi/client"

  # Models
  autoload :BaseModel,                          "qrapi/models/base_model"
  autoload :Organization,                       "qrapi/models/organization"
  autoload :Division,                           "qrapi/models/division"
  autoload :Group,                              "qrapi/models/group"
  autoload :Survey,                             "qrapi/models/survey"
  Survey.autoload :Expiration,                "qrapi/models/survey/expiration"
  autoload :User,                               "qrapi/models/user"
  autoload :LibraryMessage,                     "qrapi/models/library_message"
  autoload :Distribution,                       "qrapi/models/distribution"
  Distribution.autoload :Headers,             "qrapi/models/distribution/headers"
  Distribution.autoload :Message,             "qrapi/models/distribution/message"
  Distribution.autoload :Recipients,          "qrapi/models/distribution/recipients"
  Distribution.autoload :SurveyLink,          "qrapi/models/distribution/survey_link"
  autoload :ResponseExport,                     "qrapi/models/response_export"
  autoload :Response,                           "qrapi/models/response"
  autoload :MailingList,                        "qrapi/models/mailing_list"
  MailingList.autoload :Contact,              "qrapi/models/mailing_list/contact"

  # Resources
  autoload :OrganizationResource,               "qrapi/resources/organization_resource"
  autoload :DivisionResource,                   "qrapi/resources/division_resource"
  autoload :GroupResource,                      "qrapi/resources/group_resource"
  autoload :SurveyResource,                     "qrapi/resources/survey_resource"
  autoload :UserResource,                       "qrapi/resources/user_resource"
  autoload :LibraryMessageResource,             "qrapi/resources/library_message_resource"
  autoload :DistributionResource,               "qrapi/resources/distribution_resource"
  autoload :ResponseExportResource,             "qrapi/resources/response_export_resource"
  autoload :ResponseImportResource,             "qrapi/resources/response_import_resource"
  autoload :MailingListResource,                "qrapi/resources/mailing_list_resource"

  # JSON Maps
  autoload :OrganizationMapping,                "qrapi/mappings/organization_mapping"
  autoload :DivisionMapping,                    "qrapi/mappings/division_mapping"
  autoload :GroupMapping,                       "qrapi/mappings/group_mapping"
  autoload :SurveyMapping,                      "qrapi/mappings/survey_mapping"
  autoload :ExpirationMapping,                  "qrapi/mappings/expiration_mapping"
  autoload :UserMapping,                        "qrapi/mappings/user_mapping"
  autoload :LibraryMessageMapping,              "qrapi/mappings/library_message_mapping"
  autoload :DistributionMapping,                "qrapi/mappings/distribution_mapping"
  autoload :ResponseExportMapping,              "qrapi/mappings/response_export_mapping"
  autoload :ResponsesMapping,                   "qrapi/mappings/responses_mapping"
  autoload :MailingListMapping,                 "qrapi/mappings/mailing_list_mapping"
  autoload :ContactMapping,                     "qrapi/mappings/contact_mapping"

  # Utils
  autoload :ErrorHandlingResourceable,          "qrapi/error_handling_resourceable"
  autoload :Kartograph,                         "qrapi/utils/kartograph_optional_properties"

  # Errors
  autoload :ErrorMapping,                       "qrapi/mappings/error_mapping"
  Error = Class.new(StandardError)
  FailedCreate = Class.new(QRAPI::Error)
  FailedUpdate = Class.new(QRAPI::Error)

  class RateLimitReached < QRAPI::Error
    attr_accessor :reset_at
    attr_writer :limit, :remaining

    def limit
      @limit.to_i if @limit
    end

    def remaining
      @remaning.to_i if @remaning
    end
  end
end
